<table>{% load i18n %}
    <tr>
        <th>{% trans "Username" %}</th>
        <td>{{username}}</td>
    </tr>
    <tr>
        <th>{% trans "Name" %}</th>
        <td>{{name}}</td>
    </tr>
    <tr>
        <th>{% trans "Email" %}</th>
        <td>{{email}}</td>
    </tr>
</table>
