# Generated by Django 2.2.28 on 2022-06-01 03:27

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DataHistory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_label', models.CharField(max_length=128, verbose_name='App Label')),
                ('model_name', models.CharField(max_length=128, verbose_name='Model Name')),
                ('item_id', models.BigIntegerField(verbose_name='Item Id')),
                ('version', models.IntegerField(verbose_name='Version')),
                ('request_id', models.CharField(blank=True, max_length=128, null=True, verbose_name='Request Id')),
                ('add_time', models.DateTimeField(auto_now_add=True, verbose_name='Add Time')),
                ('username', models.CharField(blank=True, max_length=64, null=True, verbose_name='Username')),
                ('userinfo', models.TextField(blank=True, null=True, verbose_name='User Information')),
                ('action', models.CharField(choices=[('create', 'Create'), ('change', 'Change'), ('delete', 'Delete')], max_length=16, verbose_name='Action')),
                ('data', models.TextField(verbose_name='Data')),
            ],
            options={
                'verbose_name': 'Data History',
                'verbose_name_plural': 'Data Histories',
            },
        ),
    ]
